; Generate the SPI sequence to initialize the CS4227 from the Yamaha

; 18f480
; usb_programmer -c 0x300000 0000001000000000
; Crystal
; usb_programmer -c 0x300002 0000111100000110
; 4.2V brownout protection
; usb_programmer -c 0x300006 0000000010000001
; low voltage programming disabled
; usb_programmer cs4227_spi.hex
; usb_programmer -r cs4227_spi.hex




PROCESSOR 18f458
#include "p18f458.inc"


#include "pic_util.inc"


#define CLOCKSPEED 25000000
#define SPI_DELAY_VALUE 255



#define ADDRESS 0x20

#define CLK_LAT LATD
#define CLK_TRIS TRISD
#define CLK_PIN 0

#define SDO_LAT LATD
#define SDO_TRIS TRISD
#define SDO_PIN 1

#define CS_LAT LATC
#define CS_TRIS TRISC
#define CS_PIN 3

; reset on the YM3436 has to be low during startup
#define RSTN_LAT LATD
#define RSTN_TRIS TRISD
#define RSTN_PIN 2



SEND_SPI macro BYTE1, BYTE2, BYTE3
; 24 bits
	SET_REGISTER SPI_COUNTER, 24
; chip address + direction
	SET_REGISTER SPI_BUFFER + 0, BYTE1
; register address
	SET_REGISTER SPI_BUFFER + 1, BYTE2
; value
	SET_REGISTER SPI_BUFFER + 2, BYTE3
	SET_REGISTER16 SPI_STATE, spi_start
	call wait_spi
	ENDM


; flags
cblock 0x00
	INTERRUPT_COMPLETE : 1
endc


	VARSTART H'00', H'100'
	VARADD FLAGS, 1


	VARADD SPI_DELAY, 1
	VARADD SPI_STATE, 2
	VARADD SPI_BUFFER, 3
	VARADD SPI_COUNTER, 1
	VARADD STARTUP_COUNTER, 3

	VARADD TEMP, 1
	VARADD TEMP0, 1
	VARADD TEMP1, 1
	VARADD TEMP2, 1



	ORG RESETVECTOR
	goto start
	


	ORG INTVECTORHI
	goto interrupt_handler

	ORG INTVECTORLO
	goto interrupt_handler



	ORG 0x400

start:
	BANKSEL H'00'

	clrf FLAGS

	call init_spi

; lower reset
	bcf RSTN_LAT, RSTN_PIN
	bcf RSTN_TRIS, RSTN_PIN

; allow glitches
	call startup_delay

; init after glitches
	call init_spi

; hold CS
	call startup_delay


; raise reset
	bsf RSTN_LAT, RSTN_PIN
	





loop:
	clrwdt

; set attenuation for 6 channels
; need lower signal because of 5V virtual ground
	SEND_SPI ADDRESS, 0x4, 0xa
	SEND_SPI ADDRESS, 0x5, 0xa
	SEND_SPI ADDRESS, 0x6, 0xd
	SEND_SPI ADDRESS, 0x7, 0xd
	SEND_SPI ADDRESS, 0x8, 0xd
	SEND_SPI ADDRESS, 0x9, 0x17
; unmute
	SEND_SPI ADDRESS, 0x3, 0x0
; set data format
	SEND_SPI ADDRESS, 0xe, 0x3
; page 18
	SEND_SPI ADDRESS, 0x1, 0x0
	SEND_SPI ADDRESS, 0x2, 0x0


; go to sleep
loop2:
	clrwdt
	bra loop2




interrupt_handler:
	retfie FAST


startup_delay:
; startup delay
	SET_REGISTER16 STARTUP_COUNTER, 0xffff
	SET_REGISTER STARTUP_COUNTER + 2, 0x04
countdown1:
	clrwdt
	decf STARTUP_COUNTER, F
	SKIP_EQUAL_LITERAL STARTUP_COUNTER, 0xff
	bra countdown1
		decf STARTUP_COUNTER + 1, F
		SKIP_EQUAL_LITERAL STARTUP_COUNTER + 1, 0xff
		bra countdown1
			decf STARTUP_COUNTER + 2, F
			SKIP_EQUAL_LITERAL STARTUP_COUNTER + 2, 0xff
			bra countdown1
	return




init_spi:
; default values
	bcf CLK_LAT, CLK_PIN
	bsf CS_LAT, CS_PIN
	bcf SDO_LAT, SDO_PIN
; set as outputs
	bcf CLK_TRIS, CLK_PIN
	bcf CS_TRIS, CS_PIN
	bcf SDO_TRIS, SDO_PIN
	return


wait_spi:
	clrwdt
	call handle_spi
	SKIP_EQUAL_LITERAL16 SPI_STATE, spi_idle
	bra wait_spi
		return

handle_spi:
	decfsz SPI_DELAY, F
	return

		SET_REGISTER SPI_DELAY, SPI_DELAY_VALUE
		SET_PC_REG SPI_STATE



spi_idle:
	return

spi_start:
	SET_REGISTER16 SPI_STATE, spi_start2
	return

spi_start2:
; cs low
	bcf CS_LAT, CS_PIN
	SET_REGISTER16 SPI_STATE, spi_bit1
	return

spi_bit1:
	SKIP_NONZERO SPI_COUNTER
	bra spi_done

; clk low
	bcf CLK_LAT, CLK_PIN
; shift in most significant bit
	btfss SPI_BUFFER + 0, 7
		bcf SDO_LAT, SDO_PIN
	btfsc SPI_BUFFER + 0, 7
		bsf SDO_LAT, SDO_PIN
	
	rlcf SPI_BUFFER + 2, F
	rlcf SPI_BUFFER + 1, F
	rlcf SPI_BUFFER + 0, F

; decrease counter
	decf SPI_COUNTER, F
	SET_REGISTER16 SPI_STATE, spi_bit2
	return


spi_bit2:
; raise clock
	bsf CLK_LAT, CLK_PIN
	SET_REGISTER16 SPI_STATE, spi_bit1
	return

spi_done:
; lower clock
	bcf CLK_LAT, CLK_PIN
	SET_REGISTER16 SPI_STATE, spi_done2
	return

spi_done2:
; raise CS
	bcf CLK_LAT, CLK_PIN
	bsf CS_LAT, CS_PIN
	bcf SDO_LAT, SDO_PIN

	SET_REGISTER16 SPI_STATE, spi_idle
	return




END









